<?php
header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: GET, POST');

// Zeitzone explizit setzen
date_default_timezone_set('Europe/Berlin');

// Beitrittszeitpunkt aus POST-Daten lesen
$join_timestamp = isset($_POST['join_timestamp']) ? (int)$_POST['join_timestamp'] : time();
$current_time = time();
error_log("get_messages.php - join_timestamp: $join_timestamp, server time: $current_time, timezone: " . date_default_timezone_get());

// Sicherstellen, dass join_timestamp nicht in der Zukunft liegt
if ($join_timestamp > $current_time) {
    $join_timestamp = $current_time;
    error_log("get_messages.php - Adjusted join_timestamp to current time: $join_timestamp");
}

// Nachrichten aus Datei lesen (nur die letzten 50, gefiltert nach Beitrittszeitpunkt)
$messages = [];
if (file_exists('messages.txt')) {
    $all_messages = file('messages.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    $all_messages = array_slice($all_messages, -50); // Letzte 50 Nachrichten
    foreach ($all_messages as $msg) {
        $parts = explode('|', $msg, 3);
        if (count($parts) === 3) {
            // Zeitstempel in Europe/Berlin parsen
            $datetime = DateTime::createFromFormat('Y-m-d H:i:s', $parts[0], new DateTimeZone('Europe/Berlin'));
            if ($datetime !== false) {
                $msg_time = $datetime->getTimestamp();
                error_log("get_messages.php - Message: $msg, msg_time: $msg_time, join_timestamp: $join_timestamp");
                if ($msg_time >= $join_timestamp) {
                    $messages[] = $msg;
                } else {
                    error_log("get_messages.php - Message filtered out: $msg");
                }
            } else {
                error_log("get_messages.php - Invalid datetime format: $parts[0]");
            }
        } else {
            error_log("get_messages.php - Invalid message format: $msg");
        }
    }
}

// JSON-Antwort senden
echo json_encode(['messages' => $messages]);
?>
