<?php
date_default_timezone_set('Europe/Berlin'); // Zeitzone auf MESZ setzen
session_start();

// Hilfsfunktion zum Generieren einer eindeutigen ID
function generateUniqueId() {
    return md5(uniqid(rand(), true));
}

// Logout-Logik
if (isset($_POST['logout'])) {
    if (isset($_SESSION['user_id'])) {
        $userId = $_SESSION['user_id'];
        $username = $_SESSION['username'];
        $users = file_exists('user.txt') ? file('user.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) : [];
        $newUsers = [];
        foreach ($users as $user) {
            list($id, $name, $timestamp) = explode('|', $user);
            if ($id !== $userId) {
                $newUsers[] = $user;
            }
        }
        $file = fopen('user.txt', 'w');
        flock($file, LOCK_EX);
        fwrite($file, implode("\n", $newUsers) . (empty($newUsers) ? '' : "\n"));
        flock($file, LOCK_UN);
        fclose($file);

        // Automatische Nachricht für das Verlassen
        $time = date('Y-m-d H:i:s');
        $line = "$time|System|$username has left the chat\n";
        file_put_contents('messages.txt', $line, FILE_APPEND | LOCK_EX);
    }
    session_unset();
    session_destroy();
    header("Location: index.php");
    exit;
}

// Überprüfen, ob ein Benutzername gesetzt wurde
if (isset($_POST['username']) && !empty(trim($_POST['username']))) {
    $username = htmlspecialchars(trim($_POST['username']));
    $_SESSION['username'] = $username;
    $_SESSION['user_id'] = generateUniqueId();
    if (!isset($_SESSION['join_timestamp'])) {
        $_SESSION['join_timestamp'] = time(); // Beitrittszeitpunkt nur beim ersten Einloggen speichern
    }
    
    // Benutzer in user.txt speichern
    $file = fopen('user.txt', 'a');
    flock($file, LOCK_EX);
    fwrite($file, "{$_SESSION['user_id']}|$username|" . time() . "\n");
    flock($file, LOCK_UN);
    fclose($file);

    // Automatische Nachricht für das Betreten
    $time = date('Y-m-d H:i:s');
    $line = "$time|System|$username has joined the chat\n";
    file_put_contents('messages.txt', $line, FILE_APPEND | LOCK_EX);
}

// Nachrichten in Datei speichern (für AJAX-Anfragen) und Timestamp aktualisieren
if (isset($_POST['message']) && !empty(trim($_POST['message'])) && isset($_SESSION['username'])) {
    $message = htmlspecialchars(trim($_POST['message']));
    $time = date('Y-m-d H:i:s');
    $username = $_SESSION['username'];
    $line = "$time|$username|$message\n";
    file_put_contents('messages.txt', $line, FILE_APPEND | LOCK_EX);
    
    // Timestamp in user.txt aktualisieren
    if (isset($_SESSION['user_id'])) {
        $userId = $_SESSION['user_id'];
        $users = file_exists('user.txt') ? file('user.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) : [];
        $newUsers = [];
        foreach ($users as $user) {
            list($id, $name, $timestamp) = explode('|', $user);
            if ($id === $userId) {
                $newUsers[] = "$id|$name|" . time();
            } else {
                $newUsers[] = $user;
            }
        }
        $file = fopen('user.txt', 'w');
        flock($file, LOCK_EX);
        fwrite($file, implode("\n", $newUsers) . (empty($newUsers) ? '' : "\n"));
        flock($file, LOCK_UN);
        fclose($file);
    }
    
    if (isset($_POST['ajax'])) {
        header('Content-Type: application/json');
        echo json_encode(['success' => true]);
        exit;
    }
}

// Debugging: Session-Werte ausgeben
error_log("Session join_timestamp: " . (isset($_SESSION['join_timestamp']) ? $_SESSION['join_timestamp'] : 'not set'));

// Nachrichten aus Datei lesen (nur für die Initialisierung, gefiltert nach Beitrittszeitpunkt)
$messages = [];
if (file_exists('messages.txt')) {
    $join_timestamp = isset($_SESSION['join_timestamp']) ? $_SESSION['join_timestamp'] : time();
    $current_time = time();
    error_log("index.php - join_timestamp: $join_timestamp, server time: $current_time, timezone: " . date_default_timezone_get());
    // Sicherstellen, dass join_timestamp nicht in der Zukunft liegt
    if ($join_timestamp > $current_time) {
        $join_timestamp = $current_time;
        error_log("index.php - Adjusted join_timestamp to current time: $join_timestamp");
    }
    $all_messages = file('messages.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    $all_messages = array_slice($all_messages, -50); // Letzte 50 Nachrichten
    foreach ($all_messages as $msg) {
        $parts = explode('|', $msg, 3);
        if (count($parts) === 3) {
            $datetime = DateTime::createFromFormat('Y-m-d H:i:s', $parts[0], new DateTimeZone('Europe/Berlin'));
            if ($datetime !== false) {
                $msg_time = $datetime->getTimestamp();
                error_log("index.php - Message: $msg, msg_time: $msg_time, join_timestamp: $join_timestamp");
                if ($msg_time >= $join_timestamp) {
                    $messages[] = $msg;
                } else {
                    error_log("index.php - Message filtered out: $msg");
                }
            } else {
                error_log("index.php - Invalid datetime format: $parts[0]");
            }
        } else {
            error_log("index.php - Invalid message format: $msg");
        }
    }
}

// Benutzer aus Datei lesen
$users = file_exists('user.txt') ? file('user.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) : [];
?>

<!DOCTYPE html>
<html lang="de">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Simple Chat</title>
    <link href="https://fonts.googleapis.com/css2?family=Roboto:wght@400;500;700&display=swap" rel="stylesheet">
    <style>
        body {
            font-family: 'Roboto', sans-serif;
            background-color: #f0f2f5;
            margin: 0;
            padding: 20px;
            color: #333;
        }
        .main-container {
            max-width: 1200px;
            margin: 0 auto;
            position: relative;
        }
        .container {
            display: flex;
            gap: 20px;
        }
        #chatbox {
            background-color: #fff;
            border-radius: 10px;
            box-shadow: 0 4px 10px rgba(0, 0, 0, 0.1);
            padding: 20px;
            height: 400px;
            overflow-y: scroll;
            display: flex;
            flex-direction: column;
            flex: 3;
        }
        #chatbox p {
            margin: 8px 0;
            padding: 10px 15px;
            border-radius: 15px;
            max-width: 70%;
            word-wrap: break-word;
            animation: fadeIn 0.3s ease-in;
        }
        #chatbox p[data-message*="<?php echo isset($_SESSION['username']) ? htmlspecialchars($_SESSION['username']) : ''; ?>"]:not([data-message*="System"]) {
            background-color: #007bff;
            color: #fff;
            align-self: flex-end;
            margin-left: auto;
        }
        #chatbox p:not([data-message*="<?php echo isset($_SESSION['username']) ? htmlspecialchars($_SESSION['username']) : ''; ?>"]) {
            background-color: #e9ecef;
            color: #333;
            align-self: flex-start;
        }
        #chatbox p[data-message*="System"] {
            background-color: #d1e7dd;
            color: #0f5132;
            align-self: center;
            font-style: italic;
        }
        #userlist {
            background-color: #fff;
            border-radius: 10px;
            box-shadow: 0 4px 10px rgba(0, 0, 0, 0.1);
            padding: 20px;
            height: flex;
            overflow-y: scroll;
            flex: 1;
        }
        #userlist h4 {
            margin: 0 0 10px;
            color: #007bff;
            font-weight: 500;
        }
        #userlist p {
            margin: 5px 0;
            padding: 8px;
            border-radius: 5px;
            transition: background-color 0.2s;
        }
        #userlist p:hover {
            background-color: #f0f2f5;
        }
        h3 {
            color: #333;
            font-weight: 500;
            margin-bottom: 15px;
        }
        #username-form, #message-form {
            max-width: 1200px;
            margin: 20px auto;
            display: flex;
            gap: 10px;
        }
        #username-form input, #message-form input {
            flex: 1;
            padding: 12px;
            border: 1px solid #ccc;
            border-radius: 5px;
            font-size: 16px;
            transition: border-color 0.2s;
        }
        #username-form input:focus, #message-form input:focus {
            border-color: #007bff;
            outline: none;
        }
        button {
            padding: 10px 16px;
            background-color: #007bff;
            color: #fff;
            border: none;
            border-radius: 5px;
            font-size: 14px;
            cursor: pointer;
            transition: background-color 0.2s;
        }
        button:hover {
            background-color: #0056b3;
        }
        #logout-form {
            position: absolute;
            top: 0;
            right: 0;
        }
        #autoScrollContainer {
            margin: 10px 0;
            font-size: 14px;
            color: #333;
            display: flex;
            align-items: center;
        }
        #autoScroll {
            margin-right: 5px;
            cursor: pointer;
        }
        #autoScrollContainer label {
            cursor: pointer;
        }
        @keyframes fadeIn {
            from { opacity: 0; transform: translateY(10px); }
            to { opacity: 1; transform: translateY(0); }
        }
        @media (max-width: 768px) {
            .container {
                flex-direction: column;
            }
            #chatbox {
                flex: none;
                height: 50vh;
            }
            #message-form {
                margin: 10px auto;
            }
            #userlist {
                flex: none;
                height: 150px;
                margin-top: 10px;
            }
            #logout-form {
                position: static;
                margin: 10px 0;
                text-align: right;
            }
        }
    </style>
</head>
<body>
    <div class="main-container">
        <?php if (!isset($_SESSION['username'])): ?>
            <form id="username-form" method="post">
                <input type="text" name="username" placeholder="Benutzername eingeben" required>
                <button type="submit">Chat betreten</button>
            </form>
        <?php else: ?>
            <form id="logout-form" method="post">
                <button type="submit" name="logout">Logout</button>
            </form>
            <h3>Willkommen, <?php echo htmlspecialchars($_SESSION['username']); ?>!</h3>
            <div class="container">
                <div id="chatbox">
                    <?php foreach ($messages as $msg): ?>
                        <?php
                        $parts = explode('|', $msg, 3);
                        if (count($parts) === 3) {
                            $time = date('H:i:s', strtotime($parts[0]));
                            $display_msg = "[$time {$parts[1]}] {$parts[2]}";
                        } else {
                            $display_msg = $msg; // Fallback für alte Nachrichten
                        }
                        ?>
                        <p data-message="<?php echo htmlspecialchars($msg); ?>"><?php echo htmlspecialchars($display_msg); ?></p>
                    <?php endforeach; ?>
                </div>
            </div>
            <div id="autoScrollContainer">
                <input type="checkbox" id="autoScroll" checked>
                <label for="autoScroll">Automatisches Scrollen aktivieren</label>
            </div>
            <form id="message-form">
                <input type="text" name="message" placeholder="Nachricht eingeben" required>
                <button type="submit">Senden</button>
            </form>
            <div class="container">
                <div id="userlist">
                    <h4>Eingeloggte Benutzer</h4>
                    <?php foreach ($users as $user): ?>
                        <?php list($id, $name, $timestamp) = explode('|', $user); ?>
                        <p data-user-id="<?php echo htmlspecialchars($id); ?>"><?php echo htmlspecialchars($name); ?></p>
                    <?php endforeach; ?>
                </div>
            </div>
        <?php endif; ?>
    </div>
    <?php if (isset($_SESSION['username'])): ?>
        <script>
            function formatMessage(msg) {
                const parts = msg.split('|', 3);
                if (parts.length === 3) {
                    const time = new Date(parts[0]).toLocaleTimeString('de-DE', { hour: '2-digit', minute: '2-digit', second: '2-digit' });
                    return `[${time} ${parts[1]}] ${parts[2]}`;
                }
                return msg; // Fallback für alte Nachrichten
            }

            function fetchData() {
                const joinTimestamp = <?php echo isset($_SESSION['join_timestamp']) ? $_SESSION['join_timestamp'] : '0'; ?>;
                console.log('Sending joinTimestamp:', joinTimestamp); // Debugging
                fetch('get_messages.php', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
                    body: `join_timestamp=${encodeURIComponent(joinTimestamp)}`
                })
                    .then(response => response.json())
                    .then(data => {
                        console.log('Received messages:', data.messages); // Debugging
                        const chatbox = document.getElementById('chatbox');
                        const existingMessages = new Set(
                            Array.from(chatbox.getElementsByTagName('p')).map(p => p.dataset.message)
                        );

                        data.messages.forEach(msg => {
                            if (!existingMessages.has(msg)) {
                                const p = document.createElement('p');
                                p.dataset.message = msg;
                                p.textContent = formatMessage(msg);
                                chatbox.appendChild(p);
                            }
                        });

                        while (chatbox.children.length > 50) {
                            chatbox.removeChild(chatbox.firstChild);
                        }

                        if (document.getElementById('autoScroll').checked) {
                            chatbox.scrollTop = chatbox.scrollHeight;
                        }
                    })
                    .catch(error => console.error('Fehler beim Abrufen der Nachrichten:', error));

                fetch('get_users.php', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
                    body: `user_id=${encodeURIComponent('<?php echo isset($_SESSION['user_id']) ? $_SESSION['user_id'] : ''; ?>')}`
                })
                    .then(response => response.json())
                    .then(data => {
                        const userlist = document.getElementById('userlist');
                        const existingUserIds = new Set(
                            Array.from(userlist.getElementsByTagName('p')).map(p => p.dataset.userId)
                        );

                        data.users.forEach(user => {
                            if (!existingUserIds.has(user.id)) {
                                const p = document.createElement('p');
                                p.dataset.userId = user.id;
                                p.textContent = user.name;
                                userlist.appendChild(p);
                            }
                        });

                        Array.from(userlist.getElementsByTagName('p')).forEach(p => {
                            if (!data.users.some(user => user.id === p.dataset.userId)) {
                                p.remove();
                            }
                        });
                    })
                    .catch(error => console.error('Fehler beim Abrufen der Benutzer:', error));
            }

            document.getElementById('message-form').addEventListener('submit', function(e) {
                e.preventDefault();
                const messageInput = this.querySelector('input[name="message"]');
                const message = messageInput.value.trim();
                if (message) {
                    fetch('index.php', {
                        method: 'POST',
                        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
                        body: `message=${encodeURIComponent(message)}&ajax=1`
                    })
                    .then(response => response.json())
                    .then(data => {
                        if (data.success) {
                            messageInput.value = '';
                            fetchData();
                        }
                    })
                    .catch(error => console.error('Fehler beim Senden der Nachricht:', error));
                }
            });

            setInterval(fetchData, 5000);

            document.addEventListener('DOMContentLoaded', () => {
                fetchData();
                const chatbox = document.getElementById('chatbox');
                if (document.getElementById('autoScroll').checked) {
                    chatbox.scrollTop = chatbox.scrollHeight;
                }
            });
        </script>
    <?php endif; ?>
</body>
</html>
